#include <htc.h>

#include "defines.h"
#include "protos.h"
#include "globals.h"

/*
All these are for 100kHz I2C communications, and give correct timings when
viewed on an oscilloscope.
*/
//_____________________________________________________________

void send_start()	{
//assumes that SDA is input and SCL is always driven
//We simply drive the TRIS for SDA. It is assumed that
//both are high on entry, as the I2C spec requires.

	TRISC = clock_lo_data_hi;

	NOP();		//stretch the pulse
	NOP();		//stretch the pulse
	NOP();		//stretch the pulse
	NOP();		//stretch the pulse
	SCL = 1;
	NOP();		//stretch the pulse
	NOP();		//stretch the pulse
	NOP();		//stretch the pulse
	NOP();		//stretch the pulse
	TRISC &= data_lo;	//the start

	NOP();		//stretch the pulse
	NOP();		//stretch the pulse
	NOP();		//stretch the pulse
	NOP();		//stretch the pulse
	NOP();		//stretch the pulse
	NOP();		//stretch the pulse
	NOP();		//stretch the pulse
	NOP();		//stretch the pulse
	SCL = 0;
	NOP();		//stretch the pulse
	NOP();		//stretch the pulse
	NOP();		//stretch the pulse
	NOP();		//stretch the pulse
	TRISC = clock_lo_data_hi;

}
//_________________________________________________________________

void send_stop()	{
	SCL = 0;	//probably unnecessary
	NOP();		//stretch the pulse
	NOP();		//stretch the pulse
	NOP();		//stretch the pulse
	NOP();		//stretch the pulse
	TRISC &= data_lo;	//the start

	NOP();		//stretch the pulse
	NOP();		//stretch the pulse
	NOP();		//stretch the pulse
	NOP();		//stretch the pulse
	SCL = 1;
	NOP();		//stretch the pulse
	NOP();		//stretch the pulse
	NOP();		//stretch the pulse
	NOP();		//stretch the pulse
	NOP();		//stretch the pulse
	NOP();		//stretch the pulse
	NOP();		//stretch the pulse
	NOP();		//stretch the pulse
	NOP();		//stretch the pulse
	NOP();		//stretch the pulse
	NOP();		//stretch the pulse
	TRISC = clock_lo_data_hi;

	NOP();		//stretch the pulse
	NOP();		//stretch the pulse
	NOP();		//stretch the pulse
	NOP();		//stretch the pulse
	NOP();		//stretch the pulse
	NOP();		//stretch the pulse
	NOP();		//stretch the pulse
	NOP();		//stretch the pulse
	SCL = 0;

//exit with data high.
}

//_________________________________________________________________

void write_byte(unsigned char to_send)	{

unsigned char mask = 0x80;	//MSB first

	TRISC = clock_lo_data_hi;
//looping version:
	do	{
		SCL = 0;	//leave the data the same
//		NOP();		//stretch the pulse
		if((to_send & mask) == 0)
			TRISC &= data_lo;
		else
			TRISC |= data_hi;
		mask >>= 1;	//wait for the value to settle...
		NOP();		//stretch the pulse
		NOP();		//stretch the pulse
		NOP();		//stretch the pulse
		SCL = 1;	//leave the data the same
		NOP();		//stretch the pulse
		NOP();		//stretch the pulse
		NOP();		//stretch the pulse
		NOP();		//stretch the pulse
		NOP();		//stretch the pulse
	}while(mask);
//acknowledge:
	NOP();		//stretch the pulse
	SCL = 0;	//start acknowledge
	TRISC = clock_lo_data_hi;	//make sure it can be pulled low and sits high on exit
	NOP();		//stretch the pulse
	NOP();
	NOP();
	NOP();
	NOP();
	NOP();
	NOP();
	NOP();
	NOP();
	NOP();
	NOP();
	NOP();
	SCL = 1;	//clock out bit 9
	NOP();		//stretch the pulse
	NOP();
	NOP();
	NOP();		//stretch the pulse
	NOP();
	NOP();
	NOP();
	NOP();
	SCL = 0;	//finish
}
//_________________________________________________________________

unsigned char read_byte(void)	{
//simply moves the data from the sensor:
unsigned char read_buffer = 0;
unsigned char bit_mask = 0x80;
//make sure it's high_Z:

	TRISC = PORTC_SDA_IN;	//set data line to input
	SDA = 0;	//SDA is high-Z so is not sent yet.

	while(bit_mask) {
		SCL = 0;
		SCL = 0;
		SCL = 0;
		SCL = 0;
		SCL = 0;
		SCL = 0;
		SCL = 0;
		SCL = 0;
		SCL = 0;
		SCL = 1;
		SCL = 1;
		SCL = 1;
		SCL = 1;
		SCL = 1;
		SCL = 1;
		SCL = 1;
		SCL = 1;	//two usec
		if(SDA != 0)
			read_buffer |= bit_mask;	//place the bit in the buffer
		bit_mask >>= 1;	//move to next bit
	}

	SCL = 0;	//the acknowledge starts here, in fact.
	TRISC = clock_lo_data_lo;	//force data low to acknowledge
//do the acknowldege (but ignore the result!):
	SCL = 0;
	SCL = 0;
	SCL = 0;
	SCL = 0;
	SCL = 0;
	SCL = 0;
	SCL = 0;
	SCL = 1;
	SCL = 1;
	SCL = 1;
	SCL = 1;
	SCL = 1;
	SCL = 1;
	SCL = 1;
	SCL = 1;
	CLRWDT();
	SCL = 0;

	return read_buffer;
}
//___________________________________________________________________

unsigned char read_byte_no_ACK(void)	{
//simply moves the data from the sensor:
unsigned char read_buffer = 0;
unsigned char bit_mask = 0x80;
//make sure it's high_Z:

	TRISC = PORTC_SDA_IN;	//set data line to input

	while(bit_mask) {
		SCL = 0;
		SCL = 0;
		SCL = 0;
		SCL = 0;
		SCL = 0;
		SCL = 0;
		SCL = 0;
		SCL = 0;
		SCL = 0;
		SCL = 0;
		SCL = 0;
		SCL = 1;
		SCL = 1;
		SCL = 1;
		SCL = 1;
		SCL = 1;
		SCL = 1;	//two usec
		if(SDA != 0)
			read_buffer |= bit_mask;	//place the bit in the buffer
		bit_mask >>= 1;	//move to next bit
	}

	SCL = 0;	//the acknowledge starts here, in fact.
	return read_buffer;
}
//___________________________________________________________________
