#include <htc.h>
//the pin definitions have been changed to suit the LSM303D device

#define ANSELA_config		0b00000000	//no A-to-D and two comparator inputs on RA1 and RA2
#define PORTA_control_word 	0b00111111	//RA2 is high by default, so I2C can work.
#define SD0 RA2	//for the LM303D device, and the pin is pulled high by internal pull-up, so I2C
#define INT1 RA1
#define INT2 RA0

#define PORTC_control_word 	0b00100001	//for I2C, and RS232 on pin 6, RC4, and test out on pin 8 (RC2)
//SDA is on PORTC0, SCL is on PORTC1:
#define PORTC_SDA_IN PORTC_control_word

#define clock_lo_data_hi PORTC_control_word	//legacy definition!
#define PORTC_SDA_OUT		0b00011000
#define data_lo				0b11111110	//for ANDing
#define data_hi				0b00000001	//for ORing

#define clock_lo_data_lo PORTC_SDA_OUT		//ditto
#define SCL LATC1
#define SDA RC0

#define TEST_OUT LATC2	//is used to shown where the program is active (if used)

#define ANSELC_config		0b00000000	//ditto
#define APFCON0_config		0b10000000	//RX on RC5, pin 12 (not used); TX on pin 6.

#define WPUC_config			0b00011011	//just the I2C, CCP2, and INT inputs

#define DACCON0_config		0b00000000	//D-to-A disabled.
#define DACCON1_config		0b00010000	//for 2.5V

//see page 77 then 82, 2012 logbook. It's used for a timer at 1msec second per tick.
#define T6CON_config 		0b01001000	//is OFF, divide by 1, divide by 10 (see p82, 2012 logbook)
#define PR6_config 172		//for 3.528MHz Xtal. Divide input by 4 if PLL used (value tweaked from 176 for exactly 1ms)

#define	TWO_SECS 2000	//ticking at 1msec, geddit?

//These for the 4800-baud RS232 output:
#define BAUDCON_config	0b00000000	//non-inverted output, 8-bit
#define RCSTA_config 	0b10010000
#define TXSTA_config 	0b00100000	//BRGH = 0, asynchronous, enabled.
#define SPBRGL_config 31	//see page 72, 2012 logbook.
#define SPBRGH_config 0		//not used because BRGH is 1

#define OPTION_config 	0b01000111

#define PI 3.14159
#define CR 0x0D
#define LF 0x0A

//_________________________________________________________________

//These correspond with the device register names for Accelerometers:

#define SAD_WRITE	0b00111010	//The slave address ID for high SD0/SDA
#define SAD_READ	0b00111011	//for reading (after a repeated start)

#define CTRL0_ADD	0x9F	//the address of CTRL0 (0x1F) with bit 7 set for auto-increment

#define CTRL0		0b00000000	//for normal mode, filters by-passed
#define CTRL1 		0b01001111	//update after read, and all axes of acceleration enabled at 25Hz
#define CTRL2 		0b11000001	//50Hz anti-alias, +/- 2g, no self-test, (SPI 3-wire)
#define CTRL3 		0b00000000	//No INT1 actions
#define CTRL4 		0b00001000	//accelerometer data ready on INT2.
#define CTRL5		0b00001110	//No temperature, low-res magnetic, 25Hz, latch interrupt on INT2
#define CTRL6 		0b00000000	//+/-2gauss sensitivity.
#define CTRL7		0b00100000	//normal high-pass acceleration filter, no Temp, magnetic always on, continuous conversion mode
//#define STATUS_REG_A 0b00000000	//ditto?

//the first address for six bytes of magnetic data:
#define OUT_X_L_M	0x88	//the address of the low byte of the x magnetic, bit 7 set = autoincrement.
//then 9, A, B, C, D, are the high/low/high/low/high addresses of X, Y, Z magnetic sensors
#define STATUS_M	0x07
#define MASK_ZYXMDA 0b00000111

//the first address for six bytes of acceleration data:
#define OUT_X_L_A	0xA8	//bit 7 is set = auto-increment
#define STATUS_A	0x27
#define MASK_ZYXADA	0b00000111
//_________________________________________________________________

#define R2DEG 572.9577951
#define SCALE_FACTOR 1.12244898	//is 100/980 correction for the Z-axis magnetometer
#define TIME_CONSTANT 250
#define TICK_COUNT TIME_CONSTANT
//#define TICK_COUNT 5*TIME_CONSTANT

//_________________________________________________________________

