#include <htc.h>

#include "defines.h"
#include "protos.h"
#include "globals.h"

//_____________________________________________________________

void configure(void)	{	//sends the configuration bytes

	PORTC = 0;//Make sure all the port latches are low before we get going:

	send_start();
	write_byte(SAD_WRITE);	//to write to the magnetic registers.device
	write_byte(CTRL0_ADD);	//write to the control registers, autoincrementing

	write_byte(CTRL0);		//write first byte.
	write_byte(CTRL1);
	write_byte(CTRL2);
	write_byte(CTRL3);
	write_byte(CTRL4);
	write_byte(CTRL5);
	write_byte(CTRL6);
	write_byte(CTRL7);

//when we get here, we've sent all controls
	send_stop();
}
//__________________________________________________________________

unsigned char read_status_A(void)	{

unsigned char temp;
	send_start();
	write_byte(SAD_WRITE);	//to write to the accelerometer registers.
	write_byte(STATUS_A);	//no repeat
	send_start();				//repeated start for reading
	write_byte(SAD_READ);	//switch device to read from the magnetometer registers.

	temp = read_byte_no_ACK();	//all the bits, no ACK
	send_stop();

	temp &= MASK_ZYXADA;
	return temp;
}
//__________________________________________________________________

void read_accelerometers(void)	{	//sends the three magnetic configuration bytes

	PORTC = 0;//Make sure all the port latches are low before we get going:

	while(INT2 == 0)	{	//check data valid, and toggle TEST_OUT every time we go round the loop
		if(TEST_OUT == 0)
			TEST_OUT = 1;
		else
			TEST_OUT = 0;
		CLRWDT();
	}
	TEST_OUT = 0;

	send_start();
	write_byte(SAD_WRITE);	//to write to the accelerometer registers.
	write_byte(OUT_X_L_A);	//write first byte address to read from (with bit 7 set set to auto-increment).
	send_start();			//repeated start for reading
	write_byte(SAD_READ);	//switch device to read from the accelerometer registers.

	ax[0] = read_byte();	//LSByte
	ax[1] = read_byte();	//MSByte
	ay[0] = read_byte();
	ay[1] = read_byte();
	az[0] = read_byte();
	az[1] = read_byte_no_ACK();
	send_stop();

	x_acc = ax[1];
	x_acc <<= 8;
	x_acc |= ax[0];

	y_acc = ay[1];
	y_acc <<= 8;
	y_acc |= ay[0];

	z_acc = az[1];
	z_acc <<= 8;
	z_acc |= az[0];
}
//__________________________________________________________________

unsigned char read_status_M(void)	{

unsigned char temp;
	send_start();
	write_byte(SAD_WRITE);	//to write to the accelerometer registers.
	write_byte(STATUS_M);	//no repeat
	send_start();				//repeated start for reading
	write_byte(SAD_READ);	//switch device to read from the magnetometer registers.

	temp = read_byte_no_ACK();	//all the bits
	send_stop();

	temp &= MASK_ZYXMDA;
	return temp;
}
//__________________________________________________________________

void read_magnetics(void)	{	//sends the three magnetic configuration bytes

	PORTC = 0;//Make sure all the port latches are low before we get going:

	while(read_status_M() == 0)	{
		CLRWDT();
	}

	send_start();
	write_byte(SAD_WRITE);	//to write to the accelerometer registers.
	write_byte(OUT_X_L_M);		//write first byte address to read from (with bit 7 set set to auto-increment).
	send_start();				//repeated start for reading
	write_byte(SAD_READ);	//switch device to read from the magnetometer registers.

	mx[0] = read_byte();	//LSByte
	mx[1] = read_byte();	//MSByte
	my[0] = read_byte();	//N.B. CORRECT order (unlike LSM303DLHC!)
	my[1] = read_byte();	//ditto
	mz[0] = read_byte();	//ditto
	mz[1] = read_byte_no_ACK();
	send_stop();

	x_mag = mx[1];
	x_mag <<= 8;
	x_mag |= mx[0];

	y_mag = my[1];
	y_mag <<= 8;
	y_mag |= my[0];

	z_mag = mz[1];
	z_mag <<= 8;
	z_mag |= mz[0];
}
//__________________________________________________________________
